/*
 * Fasterfox is copyright 2005 by Tony Gentilcore. 
 *
 * If you wish to modify, distribute, or reproduce all
 * or part if this extension, please write me first.
 * 
 * No portion of this extension or the code contained 
 * herein this archive may be reproduced without the 
 * express written consent of the author.
 * 
 */
 
// local references to XPCOM services
var PrefetchService = Components.classes["@mozilla.org/prefetch-service;1"].getService(Components.interfaces.nsIPrefetchService);
var PreferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

// Timer global variables
var StartTime;
var StopTime;
var Finished;

// Setup the extension core
function FF_init() {	

	// because of the nature of overlays and the panel selection code,
	// this is reproduced from chrome://browser/content/pref/pref.xul, and
	// refocuses the prefsCategories window in order to ensure a clean and
	// correct selection of the right panel
	var prefsCategories = document.getElementById("prefsCategories");
	if (prefsCategories) {
 		var lastPanel = 0, button;
		try {
 			lastPanel = PreferencesService.getIntPref("browser.preferences.lastpanel");
		} 
		catch (e) {}

		prefsCategories.focus();
		button = prefsCategories.childNodes[lastPanel];
		if (button) {
			document.getElementById("panelFrame").setAttribute("src", button.getAttribute("url"));
			button.checked = true;
		}
	}
		
	// Register the web progress listener for the page load timer
	gBrowser.addProgressListener(ffListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
	
	// show or hide the timer
	if(PreferencesService.getBoolPref("extensions.fasterfox.pageLoadTimer", true)) {
		document.getElementById("ff-statusbar").setAttribute("hidden", "false");
	} else {
		document.getElementById("ff-statusbar").setAttribute("hidden", "true");
	}

  	return true;
}

// Tear down the extension
function FF_uninit() {
    gBrowser.removeProgressListener(ffListener);
}

// GetAbsoluteURL
function getAbsoluteUrl(url, docUrl) { 
	if(url && url.indexOf('://')>0) return url;
	docUrl=(docUrl)? docUrl.substring(0,docUrl.lastIndexOf('/')+1):dynapi.documentPath;
	url=url.replace(/^(.\/)*/,'');
	docUrl=docUrl.replace(/(\?.*)$/,'').replace(/(#.*)*$/,'').replace(/[^\/]*$/,'');
	if (url.indexOf('/')==0) return docUrl.substring(0,docUrl.indexOf('/',docUrl.indexOf('//')+2))+url;
	else while(url.indexOf('../')==0){
		url=url.replace(/^..\//,'');
		docUrl=docUrl.replace(/([^\/]+[\/][^\/]*)$/,'');
	};
	return docUrl+url;
}

// Converts a string URL into a URI object
function makeURI(aURL) {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    return ioService.newURI(aURL, null, null);
}

// Iterates through all the <a> links on a page and adds 
// them to the queue to be prefetched.
function doEnhancedPrefetch(aEvent) {

	// exit if enhanced prefetching is turned off
	if(!PreferencesService.getBoolPref("extensions.fasterfox.enhancedPrefetching")) {
		return;
	}
	
	// doc is document that triggered "onload" event
	var doc = aEvent.originalTarget; 

	// find all links
	var a = doc.getElementsByTagName('a');
	var href, hreflength;

	// iterate through the links
	for (i=0; i<a.length; i++) {
		href = a[i].getAttribute('href');

		// Don't prefetch secure links or links w/ query strings
		if(href.indexOf('?')>=0 || href.indexOf('https://')>=0) continue;
		
		// Don't prefetch the same page we are on
		if(href == doc.location.href) continue;
		
		// Don't prefetch things that look like a logout link
		// Special thanks to b1naryb0y and Kai Risku for this bug fix
		if(href.indexOf('logout')>=0 || href.indexOf('logoff')>=0) continue;
		
		// Only prefetch static content, this causes dynamic pages to be ignored
		fileExtensionStartPos = href.length-4;
		if(!(href.indexOf('.htm') == fileExtensionStartPos 
		  || href.indexOf('.html') == fileExtensionStartPos-1
		  || href.indexOf('.txt') == fileExtensionStartPos
		  || href.indexOf('.pdf') == fileExtensionStartPos
		  || href.indexOf('.gif') == fileExtensionStartPos
		  || href.indexOf('.jpg') == fileExtensionStartPos
		  || href.indexOf('.png') == fileExtensionStartPos)) continue;

		try {
			PrefetchService.prefetchURI(makeURI(getAbsoluteUrl(href, doc.location.href)), makeURI(doc.location.href), true);
   		} catch(e) { 
			//if(e.name != "NS_ERROR_ABORT") alert(getAbsoluteUrl(href, doc.location.href) + '\n' + e.toString()); 
		}
	}

}

// Converts a Delta date into an english string
// showMilliseconds = true, gives fractions of seconds
// showMilliseconds = false, gives whole seconds
function TimeStr(delta, showMilliseconds) {

	// Calc seconds and milliseconds
	var mseconds = delta % 1000;
	delta = (delta - mseconds) / 1000;
	var seconds = delta.toString(); 
	
	if(showMilliseconds) {
		mseconds = mseconds.toString(); 
		var pd = ''; var i;
	    if (3 > mseconds.length) { 
	   		for (i = 0; i < (3-mseconds.length); i++) { 
	        	pd += '0'; 
	    	} 
	    } 
	    mseconds = mseconds + pd;
	} else {
		mseconds = "000";
	}
	
	return(seconds + "." + mseconds + "s");
}

// Updates the display of the page load timer
function ff_updateTimer() {
	var Delta;
	if (!Finished) {
		CurrentTime = new Date();
		Delta = CurrentTime.getTime() - StartTime.getTime();
		document.getElementById("ff-label").value = TimeStr(Delta, false);
		setTimeout('ff_updateTimer()',1000);
	} else {
		Delta = StopTime.getTime() - StartTime.getTime();
		document.getElementById("ff-label").value = TimeStr(Delta, true);
	}
}

// Ends page load timer
function ff_stopTimer() {
	// Stop timer
	StopTime = new Date();
	Finished = true;
	ff_updateTimer();	
}

// Starts page load timer
function ff_startTimer() {
	// Start timer  
	StartTime = new Date();
	Finished = false;
	ff_updateTimer();	
}

// Show or hide the page load timer
// if show = true, it will be displayed
// if show = false, it will be hidden
function ff_showTimer(show) {
	PreferencesService.setBoolPref("extensions.fasterfox.pageLoadTimer", show);	
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    var win = wm.getMostRecentWindow("navigator:browser");
    if (win) {
		win.document.getElementById("ff-statusbar").setAttribute("hidden", !show);
		win.document.getElementById("ff-label").value = "Fasterfox";
   }
}

// Clears the browser disk and memory cache
function ff_clearCache() {
	var cacheService = Components.classes["@mozilla.org/network/cache-service;1"]
                                 .getService(Components.interfaces.nsICacheService);
        cacheService.evictEntries(Components.interfaces.nsICache.STORE_ON_DISK);
        cacheService.evictEntries(Components.interfaces.nsICache.STORE_IN_MEMORY);
}